import java.util.Random;

/**
 * A basic class to handle dealing and shuffling cards from one or more decks of
 * cards.
 * 
 * @author CS 201 Instructors
 * 
 */
public class CardDealer {

  private FaceUpCard[] cards; // all of the cards to be dealt
  private int position; // current position in our deck of cards

  /**
   * Creates a new card dealer with numDecks decks in the deck.
   * 
   * @param numDecks the number of card decks to use
   */
  public CardDealer(int numDecks) {
    int numValues = FaceUpCard.MAX_VALUE - FaceUpCard.MIN_VALUE + 1;
    int numSuits = FaceUpCard.SUITS.length;
    int numCards = numDecks * numValues * numSuits;
    cards = new FaceUpCard[numCards];

    int cardIndex = 0;
    for (int i = 0; i < numDecks; i++) {
      for (int suitIndex = 0; suitIndex < FaceUpCard.SUITS.length; suitIndex++) {
        for (int valueIndex = FaceUpCard.MIN_VALUE; valueIndex <= FaceUpCard.MAX_VALUE; valueIndex++) {
          cards[cardIndex] = new FaceUpCard(FaceUpCard.SUITS[suitIndex], valueIndex);
          cardIndex++;
        }
      }
    }

    shuffle();
  }

  /**
   * @return the next card from the deck.
   * 
   * @pre assumes there are cards left in the deck.
   */
  public FaceUpCard draw() {
    position++;
    return cards[position - 1];
  }

  /**
   * Randomly shuffles the deck and resets the card to be dealt as the first card.
   */
  public void shuffle() {
    Random rand = new Random();
    for (int i = 0; i < cards.length; i++) {
      int nextPosition = rand.nextInt(cards.length);

      // swap this card with the card at next position
      FaceUpCard temp = cards[i];
      cards[i] = cards[nextPosition];
      cards[nextPosition] = temp;
    }
    position = 0;
  }

  /**
   * Tests the CardDealer class.
   */
  public static void main(String[] args) {
    CardDealer dealer = new CardDealer(2);
    for (int i = 0; i < 100; i++) {
      System.out.println(dealer.draw());
    }
  }
}